/*
 * CINELERRA
 * Copyright (C) 2016 Eric Olson
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef YUV411WIN_H
#define YUV411WIN_H

#include "filexml.inc"
#include "yuv411.h"
#include "mutex.h"
#include "pluginvclient.h"
#include "bctitle.h"
#include "bccolors.h"
#include "fonts.h"

class yuv411Toggle;
class yuv411Offset;
class yuv411Thresh;
class yuv411Bias;
class yuv411Reset;

class yuv411Window : public PluginClientWindow
{
public:
	yuv411Window(yuv411Main *client);
	~yuv411Window();

	void create_objects();
	void update();
	int close_event();
	void update_enables();
	void show_warning(int warn);

	yuv411Main *client;
	yuv411Toggle *avg_vertical;
	yuv411Toggle *int_horizontal;
	yuv411Toggle *inpainting;
	yuv411Offset *offset;
	yuv411Thresh *thresh;
	yuv411Bias *bias;
	BC_Title *yuv_warning;
	int colormodel;
	yuv411Reset *reset;
};

class yuv411Toggle : public BC_CheckBox
{
public:
	yuv411Toggle(yuv411Main *client, int *output, char *string, int x, int y);
	~yuv411Toggle();
	int handle_event();

	yuv411Main *client;
	int *output;
};

class yuv411Offset : public BC_FSlider
{
public:
	yuv411Offset(yuv411Main *client, int x, int y);
	int handle_event();
	yuv411Main *client;
};

class yuv411Thresh : public BC_FSlider
{
public:
	yuv411Thresh(yuv411Main *client, int x, int y);
	int handle_event();
	yuv411Main *client;
};

class yuv411Bias : public BC_FSlider
{
public:
	yuv411Bias(yuv411Main *client, int x, int y);
	int handle_event();
	yuv411Main *client;
};

class yuv411Reset : public BC_GenericButton
{
public:
	yuv411Reset(yuv411Main *client, yuv411Window *window, int x, int y);
	~yuv411Reset();
	int handle_event();
	yuv411Main *client;
	yuv411Window *window;
};

#endif
